//----------------------------
// CS 315 Lighting Lab part 2
//
// Modified by Alex Clarke 2010, 2012, 2015
//
//
// The provided shader implements one positional light similar to
// the classic compatibility mode
//
// Purpose:
//   - investigate difference between position and directional lights
//     and how to switch between the two with the provided shader
//   - move a positional light source
//   - experiment with light and material properties
//   - implement attenuation
//----------------------------
#include "Angel.h"
#include "uofrGraphics.h"


//For convenience
typedef Angel::vec4 point4;

//Shader Programs
GLuint program1;

//Transformation Uniforms for Shader
GLint mvIndex;
GLint pIndex;

//Transformation Matrices
mat4 p, mv;

//Lighting Uniforms for Shader
GLint iLightPosition;
GLint ambientProduct;
GLint diffuseProduct;
GLint specularProduct;
GLint emission;
GLint shininess;
GLint lighting;
GLint uColor;

//Initial or "Default" Lighting Values
vec4 LightPosition(0.0f,0.0f,10.0f,0.0f);
vec4 LightAmbient(0.2f,0.2f,0.2f,1.0f);
vec4 LightDiffuse(1.0f,1.0f,1.0f,1.0f);
vec4 LightSpecular(1.0f,1.0f,1.0f,1.0f);

//Initial or "Default" Material Properties
vec4 MatAmbient(0.2f,0.2f,0.2f,1.0f);
vec4 MatDiffuse(0.8f,0.8f,0.8f,1.0f);
vec4 MatSpecular(0.0f,0.0f,0.0f,1.0f);
vec4 MatEmission(0.0f,0.0f,0.0f,1.0f);
GLfloat MatShininess = 5.0f;

//Number of vertices in Torus
GLuint ntVerts;

//Light rotation variable
GLint spin = 0;


//----------------------------------------------------------------------------

/* Initialize VAOs, shaders, material properties, lighting properties, and 
 * depth buffer.
 */
void init()
{	
	//Set up shaders and VAOs
	program1 = InitShader( "vShaderL4.glsl", "fShaderL4.glsl" );

	glUseProgram( program1 );

    //Connect uofrGraphics to the shader
    urgl.connectShader(program1, "vPosition", "vNormal", "none");
    
	//Get locations of transformation matrices from shader
 	mvIndex = glGetUniformLocation(program1, "mv");
	pIndex = glGetUniformLocation(program1, "p");

	//Get locations of lighting uniforms from shader
	iLightPosition = glGetUniformLocation(program1, "lightPosition");
	ambientProduct = glGetUniformLocation(program1, "ambientProduct");
	diffuseProduct = glGetUniformLocation(program1, "diffuseProduct");
	specularProduct = glGetUniformLocation(program1, "specularProduct");
	emission = glGetUniformLocation(program1, "emission");
	shininess = glGetUniformLocation(program1, "shininess");
	lighting = glGetUniformLocation(program1, "lighting");
	uColor = glGetUniformLocation(program1, "uColor");

	//Set default lighting and material properties in shader.
	glUniform4fv(iLightPosition, 1, LightPosition);
	glUniform4fv(ambientProduct, 1, MatAmbient*LightAmbient);
	glUniform4fv(diffuseProduct, 1, MatDiffuse*LightDiffuse);
	glUniform4fv(specularProduct, 1, MatSpecular*LightSpecular);
	glUniform4fv(emission, 1, MatEmission);
	glUniform1f(shininess, MatShininess);
	glUniform1f(lighting, GL_TRUE);
	glUniform4f(uColor, 1.0f, 1.0f, 1.0f, 1.0f);

    
    glEnable(GL_DEPTH_TEST);
}

//----------------------------------------------------------------------------

/* Here the light position is modified by a modeling transformation stored in
 * the mat4 called light which is based on a rotated version of the mv matrix.
 * This places the light at a position in world coordinates.  A cube
 * represents the position of the light. 
 */

void display( void )
{
	//Clear the drawing buffer
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	mv = mat4();

	//set up viewing transformation
	point4 eye(0,0,5,1);
	point4 at(0,0,0,1);
	vec4   up(0,1,0,0);

	//mv will represent the center of the scene
	mv *= LookAt(eye, at, up);

	//Position light relative to scene
	mat4 light = mv;
	light *= RotateX( spin );
    vec4 position( 0.0, 0.0, 1.5, 1.0 );
	glUniform4fv(iLightPosition, 1,  light*position);

	//Draw small unlit wire cube at light's position
	light *= Translate(0,0,1.5);					//Move to light's position
	light *= Scale(0.1f, 0.1f, 0.1f);				//Make box small
	glUniformMatrix4fv(mvIndex, 1, GL_TRUE, light); //Apply transformations
	glUniform1f(lighting, GL_FALSE);				//Turn off lighting
	glUniform4f(uColor, 1.0f, 1.0f, 1.0f, 1.0f);	//Set the unlit color
    urgl.drawWireCube(1);
    glUniform1f(lighting, GL_TRUE);					//Turn on lighting

	//Draw Torus at center of scene
	glUniformMatrix4fv(mvIndex, 1, GL_TRUE, mv);	//Reset to center of scene
    urgl.drawSolidTorus(0.275f, 0.85f, 10, 20);

	//Show what was drawn
	glutSwapBuffers();
}

//----------------------------------------------------------------------------

void reshape (int w, int h)
{
	GLfloat aspect = GLfloat(w)/GLfloat(h);
	glViewport(0, 0, (GLsizei) w, (GLsizei) h);
	p = mat4();
	p *= Perspective(40.0f, aspect,  1.0f, 20.0f);
	glUniformMatrix4fv(pIndex, 1, GL_TRUE, p);
}


void keyboard( unsigned char key, int x, int y )
{
	switch (key)
	{
	case '1':
		//Switch to vertex based lighting
		break;
	case '2':
		//Switch to fragment based lighting
		break;

	case 033:  // Escape key
	case 'q': case 'Q':
		exit( EXIT_SUCCESS );
		break;
	}
}

//----------------------------------------------------------------------------

void mouse(int button, int state, int x, int y)
{
   switch (button) {
      case GLUT_LEFT_BUTTON:
         if (state == GLUT_DOWN) {
            spin = (spin + 30) % 360;
            glutPostRedisplay();
         }
         break;
      default:
         break;
   }
}

//----------------------------------------------------------------------------

int main( int argc, char **argv )
{
	glutInit( &argc, argv );
	glutInitWindowSize( 512, 512 );
    
#ifdef __APPLE__
	glutInitDisplayMode( GLUT_3_2_CORE_PROFILE | GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH );
#else
	// If you are using freeglut, the next two lines will check if
	// the code is truly 3.2. Otherwise, comment them out
	glutInitContextVersion( 3, 2 );
	glutInitContextProfile( GLUT_CORE_PROFILE );
	glutInitDisplayMode( GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH );
    
#endif
    
    glutCreateWindow( "Lab 4 Lighting Exercise" );
    glewExperimental = GL_TRUE;
	glewInit();

	init();
	glutDisplayFunc( display );
	glutKeyboardFunc( keyboard );
	glutReshapeFunc( reshape );
    glutMouseFunc(mouse);

	glutMainLoop();
	return 0;
}
